@@ -9,130 +9,62 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1996-2017 Insecure.Com LLC ("The Nmap  *
+ * The Nmap Security Scanner is (C) 1996-2020 Insecure.Com LLC ("The Nmap  *
  * Project"). Nmap is also a registered trademark of the Nmap Project.     *
- * This program is free software; you may redistribute and/or modify it    *
- * under the terms of the GNU General Public License as published by the   *
- * Free Software Foundation; Version 2 ("GPL"), BUT ONLY WITH ALL OF THE   *
- * CLARIFICATIONS AND EXCEPTIONS DESCRIBED HEREIN.  This guarantees your   *
- * right to use, modify, and redistribute this software under certain      *
- * conditions.  If you wish to embed Nmap technology into proprietary      *
- * software, we sell alternative licenses (contact sales@nmap.com).        *
- * Dozens of software vendors already license Nmap technology such as      *
- * host discovery, port scanning, OS detection, version detection, and     *
- * the Nmap Scripting Engine.                                              *
  *                                                                         *
- * Note that the GPL places important restrictions on "derivative works",  *
- * yet it does not provide a detailed definition of that term.  To avoid   *
- * misunderstandings, we interpret that term as broadly as copyright law   *
- * allows.  For example, we consider an application to constitute a        *
- * derivative work for the purpose of this license if it does any of the   *
- * following with any software or content covered by this license          *
- * ("Covered Software"):                                                   *
+ * This program is distributed under the terms of the Nmap Public Source   *
+ * License (NPSL). The exact license text applying to a particular Nmap    *
+ * release or source code control revision is contained in the LICENSE     *
+ * file distributed with that version of Nmap or source code control       *
+ * revision. More Nmap copyright/legal information is available from       *
+ * https://nmap.org/book/man-legal.html, and further information on the    *
+ * NPSL license itself can be found at https://nmap.org/npsl. This header  *
+ * summarizes some key points from the Nmap license, but is no substitute  *
+ * for the actual license text.                                            *
  *                                                                         *
- * o Integrates source code from Covered Software.                         *
+ * Nmap is generally free for end users to download and use themselves,    *
+ * including commercial use. It is available from https://nmap.org.        *
  *                                                                         *
- * o Reads or includes copyrighted data files, such as Nmap's nmap-os-db   *
- * or nmap-service-probes.                                                 *
+ * The Nmap license generally prohibits companies from using and           *
+ * redistributing Nmap in commercial products, but we sell a special Nmap  *
+ * OEM Edition with a more permissive license and special features for     *
+ * this purpose. See https://nmap.org/oem                                  *
  *                                                                         *
- * o Is designed specifically to execute Covered Software and parse the    *
- * results (as opposed to typical shell or execution-menu apps, which will *
- * execute anything you tell them to).                                     *
+ * If you have received a written Nmap license agreement or contract       *
+ * stating terms other than these (such as an Nmap OEM license), you may   *
+ * choose to use and redistribute Nmap under those terms instead.          *
  *                                                                         *
- * o Includes Covered Software in a proprietary executable installer.  The *
- * installers produced by InstallShield are an example of this.  Including *
- * Nmap with other software in compressed or archival form does not        *
- * trigger this provision, provided appropriate open source decompression  *
- * or de-archiving software is widely available for no charge.  For the    *
- * purposes of this license, an installer is considered to include Covered *
- * Software even if it actually retrieves a copy of Covered Software from  *
- * another source during runtime (such as by downloading it from the       *
- * Internet).                                                              *
- *                                                                         *
- * o Links (statically or dynamically) to a library which does any of the  *
- * above.                                                                  *
- *                                                                         *
- * o Executes a helper program, module, or script to do any of the above.  *
- *                                                                         *
- * This list is not exclusive, but is meant to clarify our interpretation  *
- * of derived works with some common examples.  Other people may interpret *
- * the plain GPL differently, so we consider this a special exception to   *
- * the GPL that we apply to Covered Software.  Works which meet any of     *
- * these conditions must conform to all of the terms of this license,      *
- * particularly including the GPL Section 3 requirements of providing      *
- * source code and allowing free redistribution of the work as a whole.    *
- *                                                                         *
- * As another special exception to the GPL terms, the Nmap Project grants  *
- * permission to link the code of this program with any version of the     *
- * OpenSSL library which is distributed under a license identical to that  *
- * listed in the included docs/licenses/OpenSSL.txt file, and distribute   *
- * linked combinations including the two.                                  *
- *                                                                         *
- * The Nmap Project has permission to redistribute Npcap, a packet         *
- * capturing driver and library for the Microsoft Windows platform.        *
- * Npcap is a separate work with it's own license rather than this Nmap    *
- * license.  Since the Npcap license does not permit redistribution        *
- * without special permission, our Nmap Windows binary packages which      *
- * contain Npcap may not be redistributed without special permission.      *
- *                                                                         *
- * Any redistribution of Covered Software, including any derived works,    *
- * must obey and carry forward all of the terms of this license, including *
- * obeying all GPL rules and restrictions.  For example, source code of    *
- * the whole work must be provided and free redistribution must be         *
- * allowed.  All GPL references to "this License", are to be treated as    *
- * including the terms and conditions of this license text as well.        *
- *                                                                         *
- * Because this license imposes special exceptions to the GPL, Covered     *
- * Work may not be combined (even as part of a larger work) with plain GPL *
- * software.  The terms, conditions, and exceptions of this license must   *
- * be included as well.  This license is incompatible with some other open *
- * source licenses as well.  In some cases we can relicense portions of    *
- * Nmap or grant special permissions to use it in other open source        *
- * software.  Please contact fyodor@nmap.org with any such requests.       *
- * Similarly, we don't incorporate incompatible open source software into  *
- * Covered Software without special permission from the copyright holders. *
- *                                                                         *
- * If you have any questions about the licensing restrictions on using     *
- * Nmap in other works, are happy to help.  As mentioned above, we also    *
- * offer alternative license to integrate Nmap into proprietary            *
- * applications and appliances.  These contracts have been sold to dozens  *
- * of software vendors, and generally include a perpetual license as well  *
- * as providing for priority support and updates.  They also fund the      *
- * continued development of Nmap.  Please email sales@nmap.com for further *
- * information.                                                            *
- *                                                                         *
- * If you have received a written license agreement or contract for        *
- * Covered Software stating terms other than these, you may choose to use  *
- * and redistribute Covered Software under those terms instead of these.   *
+ * The official Nmap Windows builds include the Npcap software             *
+ * (https://npcap.org) for packet capture and transmission. It is under    *
+ * separate license terms which forbid redistribution without special      *
+ * permission. So the official Nmap Windows builds may not be              *
+ * redistributed without special permission (such as an Nmap OEM           *
+ * license).                                                               *
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
  * right to know exactly what a program is going to do before they run it. *
  * This also allows you to audit the software for security holes.          *
  *                                                                         *
  * Source code also allows you to port Nmap to new platforms, fix bugs,    *
- * and add new features.  You are highly encouraged to send your changes   *
- * to the dev@nmap.org mailing list for possible incorporation into the    *
- * main distribution.  By sending these changes to Fyodor or one of the    *
- * Insecure.Org development mailing lists, or checking them into the Nmap  *
- * source code repository, it is understood (unless you specify            *
- * otherwise) that you are offering the Nmap Project the unlimited,        *
- * non-exclusive right to reuse, modify, and relicense the code.  Nmap     *
- * will always be available Open Source, but this is important because     *
- * the inability to relicense code has caused devastating problems for     *
- * other Free Software projects (such as KDE and NASM).  We also           *
- * occasionally relicense the code to third parties as discussed above.    *
- * If you wish to specify special license conditions of your               *
- * contributions, just say so when you send them.                          *
+ * and add new features.  You are highly encouraged to submit your         *
+ * changes as a Github PR or by email to the dev@nmap.org mailing list     *
+ * for possible incorporation into the main distribution. Unless you       *
+ * specify otherwise, it is understood that you are offering us very       *
+ * broad rights to use your submissions as described in the Nmap Public    *
+ * Source License Contributor Agreement. This is important because we      *
+ * fund the project by selling licenses with various terms, and also       *
+ * because the inability to relicense code has caused devastating          *
+ * problems for other Free Software projects (such as KDE and NASM).       *
  *                                                                         *
- * This program is distributed in the hope that it will be useful, but     *
- * WITHOUT ANY WARRANTY; without even the implied warranty of              *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the Nmap      *
- * license file for more details (it's in a COPYING file included with     *
- * Nmap, and also available from https://svn.nmap.org/nmap/COPYING)        *
+ * The free version of Nmap is distributed in the hope that it will be     *
+ * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of  *
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. Warranties,        *
+ * indemnification and commercial support are all available through the    *
+ * Npcap OEM program--see https://nmap.org/oem.                            *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: output.cc 36788 2017-06-07 12:32:38Z dmiller $ */
+/* $Id: output.cc 38078 2020-10-02 16:12:22Z dmiller $ */
 
 #include "nmap.h"
 #include "output.h"
@@ -151,6 +83,7 @@
 #include "xml.h"
 #include "nbase.h"
 #include "libnetutil/netutil.h"
+#include <nsock.h>
 
 #include <math.h>
 
@@ -309,16 +242,21 @@ void win32_fatal_raw_sockets(const char *devname) {
    words the names of interfaces Nmap has no way of using.*/
 static void print_iflist_pcap_mapping(const struct interface_info *iflist,
                                       int numifs) {
-  pcap_if_t *pcap_ifs;
+  pcap_if_t *pcap_ifs = NULL;
+  char errbuf[PCAP_ERRBUF_SIZE];
   std::list<const pcap_if_t *> leftover_pcap_ifs;
   std::list<const pcap_if_t *>::iterator leftover_p;
   int i;
 
   /* Build a list of "leftover" libpcap interfaces. Initially it contains all
      the interfaces. */
-  pcap_ifs = getpcapinterfaces();
-  for (const pcap_if_t *p = pcap_ifs; p != NULL; p = p->next)
-    leftover_pcap_ifs.push_front(p);
+  if (o.have_pcap) {
+    if (pcap_findalldevs(&pcap_ifs, errbuf) == -1) {
+      fatal("pcap_findalldevs(): Cannot retrieve pcap interfaces: %s", errbuf);
+    }
+    for (const pcap_if_t *p = pcap_ifs; p != NULL; p = p->next)
+      leftover_pcap_ifs.push_front(p);
+  }
 
   if (numifs > 0 || !leftover_pcap_ifs.empty()) {
     NmapOutputTable Tbl(1 + numifs + leftover_pcap_ifs.size(), 2);
@@ -362,7 +300,9 @@ static void print_iflist_pcap_mapping(const struct interface_info *iflist,
     log_flush_all();
   }
 
-  pcap_freealldevs(pcap_ifs);
+  if (pcap_ifs) {
+    pcap_freealldevs(pcap_ifs);
+  }
 }
 #endif
 
@@ -489,17 +429,30 @@ static std::string escape_for_screen(const std::string s) {
    xml_write_escaped is not enough; some characters are not allowed to appear in
    XML, not even escaped. */
 std::string protect_xml(const std::string s) {
-  /* escape_for_screen is good enough. */
-  return escape_for_screen(s);
+  std::string r;
+
+  for (unsigned int i = 0; i < s.size(); i++) {
+    char buf[5];
+    unsigned char c = s[i];
+    // Printable and some whitespace ok.
+    if (c == '\t' || c == '\r' || c == '\n' || (0x20 <= c && c <= 0x7e)) {
+      r += c;
+    } else {
+      Snprintf(buf, sizeof(buf), "\\x%02X", c);
+      r += buf;
+    }
+  }
+
+  return r;
 }
 
 /* This is a helper function to determine the ordering of the script results
    based on their id. */
-static bool scriptid_lessthan(ScriptResult a, ScriptResult b) {
+static bool scriptid_lessthan(const ScriptResult &a, const ScriptResult &b) {
   return strcmp(a.get_id(), b.get_id()) < 0;
 }
 
-static char *formatScriptOutput(ScriptResult sr) {
+static char *formatScriptOutput(const ScriptResult &sr) {
   std::vector<std::string> lines;
 
   std::string c_output;
@@ -626,18 +579,30 @@ void printportoutput(Target *currenths, PortList *plist) {
     return;
   }
 
-  if (o.verbose > 1 || o.debugging) {
+  if ((o.verbose > 1 || o.debugging) && currenths->StartTime()) {
     time_t tm_secs, tm_sece;
-    struct tm *tm;
+    struct tm tm;
+    int err;
     char tbufs[128];
     tm_secs = currenths->StartTime();
     tm_sece = currenths->EndTime();
-    tm = localtime(&tm_secs);
-    if (strftime(tbufs, sizeof(tbufs), "%Y-%m-%d %H:%M:%S %Z", tm) <= 0)
-      fatal("Unable to properly format host start time");
-
-    log_write(LOG_PLAIN, "Scanned at %s for %lds\n",
-              tbufs, (long) (tm_sece - tm_secs));
+    err = n_localtime(&tm_secs, &tm);
+    if (err) {
+      error("Error in localtime: %s", strerror(err));
+      log_write(LOG_PLAIN, "Scanned for %lds\n",
+          (long) (tm_sece - tm_secs));
+    }
+    else {
+      if (strftime(tbufs, sizeof(tbufs), "%Y-%m-%d %H:%M:%S %Z", &tm) <= 0) {
+        error("Unable to properly format host start time");
+        log_write(LOG_PLAIN, "Scanned for %lds\n",
+            (long) (tm_sece - tm_secs));
+      }
+      else {
+        log_write(LOG_PLAIN, "Scanned at %s for %lds\n",
+            tbufs, (long) (tm_sece - tm_secs));
+      }
+    }
   }
   log_write(LOG_MACHINE, "Host: %s (%s)", currenths->targetipstr(),
             currenths->HostName());
@@ -718,7 +683,7 @@ void printportoutput(Target *currenths, PortList *plist) {
           first = 0;
         if (o.reason) {
           if (current->reason.ttl)
-            Tbl->addItemFormatted(rowno, reasoncol, false, "%s ttl %d", 
+            Tbl->addItemFormatted(rowno, reasoncol, false, "%s ttl %d",
                                 port_reason_str(current->reason), current->reason.ttl);
           else
             Tbl->addItem(rowno, reasoncol, true, port_reason_str(current->reason));
@@ -783,7 +748,7 @@ void printportoutput(Target *currenths, PortList *plist) {
         Tbl->addItem(rowno, servicecol, true, serviceinfo);
         if (o.reason) {
           if (current->reason.ttl)
-            Tbl->addItemFormatted(rowno, reasoncol, false, "%s ttl %d", 
+            Tbl->addItemFormatted(rowno, reasoncol, false, "%s ttl %d",
                                   port_reason_str(current->reason), current->reason.ttl);
           else
             Tbl->addItem(rowno, reasoncol, true, port_reason_str(current->reason));
@@ -946,6 +911,9 @@ char *logfilename(const char *str, struct tm *tm) {
       case 'D':
         strftime(tbuf, sizeof tbuf, "%m%d%y", tm);
         break;
+      case 'F':
+        strftime(tbuf, sizeof tbuf, "%Y-%m-%d", tm);
+        break;
       default:
         *p++ = *str;
         continue;
@@ -1112,9 +1080,9 @@ void log_flush_all() {
 }
 
 /* Open a log descriptor of the type given to the filename given.  If
-   append is nonzero, the file will be appended instead of clobbered if
+   append is true, the file will be appended instead of clobbered if
    it already exists.  If the file does not exist, it will be created */
-int log_open(int logt, int append, char *filename) {
+int log_open(int logt, bool append, char *filename) {
   int i = 0;
   if (logt <= 0 || logt > LOG_FILE_MASK)
     return -1;
@@ -1360,6 +1328,14 @@ static void write_xml_initial_hostinfo(Target *currenths,
   log_flush_all();
 }
 
+void write_xml_hosthint(Target *currenths) {
+  xml_start_tag("hosthint");
+  write_xml_initial_hostinfo(currenths, (currenths->flags & HOST_UP) ? "up" : "down");
+  xml_end_tag();
+  xml_newline();
+  log_flush_all();
+}
+
 static void write_xml_osclass(const OS_Classification *osclass, double accuracy) {
   xml_open_start_tag("osclass");
   xml_attribute("type", "%s", osclass->Device_Type);
@@ -1431,7 +1407,7 @@ static char *num_to_string_sigdigits(double d, int digits) {
 /* Writes a heading for a full scan report ("Nmap scan report for..."),
    including host status and DNS records. */
 void write_host_header(Target *currenths) {
-  if ((currenths->flags & HOST_UP) || o.verbose || o.resolve_all) {
+  if ((currenths->flags & HOST_UP) || o.verbose || o.always_resolve) {
     if (currenths->flags & HOST_UP) {
       log_write(LOG_PLAIN, "Nmap scan report for %s\n", currenths->NameIP());
     } else if (currenths->flags & HOST_DOWN) {
@@ -1443,17 +1419,14 @@ void write_host_header(Target *currenths) {
   }
   write_host_status(currenths);
   if (currenths->TargetName() != NULL
-      && currenths->resolved_addrs.size() > 1) {
-    const struct sockaddr_storage *hs_ss = currenths->TargetSockAddr();
+      && !currenths->unscanned_addrs.empty()) {
 
     log_write(LOG_PLAIN, "Other addresses for %s (not scanned):",
       currenths->TargetName());
-    for (std::list<struct sockaddr_storage>::const_iterator it = currenths->resolved_addrs.begin(), end = currenths->resolved_addrs.end();
+    for (std::list<struct sockaddr_storage>::const_iterator it = currenths->unscanned_addrs.begin(), end = currenths->unscanned_addrs.end();
         it != end; it++) {
       struct sockaddr_storage ss = *it;
-      if (!sockaddr_storage_equal(&ss, hs_ss)) {
-        log_write(LOG_PLAIN, " %s", inet_ntop_ez(&ss, sizeof(ss)));
-      }
+      log_write(LOG_PLAIN, " %s", inet_ntop_ez(&ss, sizeof(ss)));
     }
     log_write(LOG_PLAIN, "\n");
   }
@@ -2015,17 +1988,23 @@ void printosscanoutput(Target *currenths) {
     char tmbuf[128];
     struct timeval tv;
     double uptime;
-    strncpy(tmbuf, ctime(&currenths->seq.lastboot), sizeof(tmbuf));
+    int err = n_ctime(tmbuf, sizeof(tmbuf), &currenths->seq.lastboot);
     chomp(tmbuf);
     gettimeofday(&tv, NULL);
     uptime = difftime(tv.tv_sec, currenths->seq.lastboot);
-    if (o.verbose)
-      log_write(LOG_PLAIN, "Uptime guess: %.3f days (since %s)\n",
+    if (o.verbose) {
+      if (err)
+        log_write(LOG_PLAIN, "Uptime guess: %.3f days\n",
+                uptime / 86400);
+      else
+        log_write(LOG_PLAIN, "Uptime guess: %.3f days (since %s)\n",
                 uptime / 86400,
                 tmbuf);
+    }
     xml_open_start_tag("uptime");
     xml_attribute("seconds", "%.0f", uptime);
-    xml_attribute("lastboot", "%s", tmbuf);
+    if (!err)
+      xml_attribute("lastboot", "%s", tmbuf);
     xml_close_empty_tag();
     xml_newline();
   }
@@ -2107,7 +2086,7 @@ void printosscanoutput(Target *currenths) {
   log_flush_all();
 }
 
-/* An auxillary function for printserviceinfooutput(). Returns
+/* An auxiliary function for printserviceinfooutput(). Returns
    non-zero if a and b are considered the same hostnames. */
 static int hostcmp(const char *a, const char *b) {
   return strcasecmp(a, b) == 0;
@@ -2500,20 +2479,30 @@ void printStatusMessage() {
    You have to close the tag with xml_close_empty_tag. */
 void print_xml_finished_open(time_t timep, const struct timeval *tv) {
   char mytime[128];
+  int err = n_ctime(mytime, sizeof(mytime), &timep);
 
-  Strncpy(mytime, ctime(&timep), sizeof(mytime));
   chomp(mytime);
 
   xml_open_start_tag("finished");
   xml_attribute("time", "%lu", (unsigned long) timep);
-  xml_attribute("timestr", "%s", mytime);
+  if (!err) {
+    xml_attribute("timestr", "%s", mytime);
+    xml_attribute("summary",
+        "Nmap done at %s; %u %s (%u %s up) scanned in %.2f seconds",
+        mytime, o.numhosts_scanned,
+        (o.numhosts_scanned == 1) ? "IP address" : "IP addresses",
+        o.numhosts_up, (o.numhosts_up == 1) ? "host" : "hosts",
+        o.TimeSinceStart(tv));
+  }
+  else {
+    xml_attribute("summary",
+        "Nmap done; %u %s (%u %s up) scanned in %.2f seconds",
+        o.numhosts_scanned,
+        (o.numhosts_scanned == 1) ? "IP address" : "IP addresses",
+        o.numhosts_up, (o.numhosts_up == 1) ? "host" : "hosts",
+        o.TimeSinceStart(tv));
+  }
   xml_attribute("elapsed", "%.2f", o.TimeSinceStart(tv));
-  xml_attribute("summary",
-    "Nmap done at %s; %u %s (%u %s up) scanned in %.2f seconds",
-    mytime, o.numhosts_scanned,
-    (o.numhosts_scanned == 1) ? "IP address" : "IP addresses",
-    o.numhosts_up, (o.numhosts_up == 1) ? "host" : "hosts",
-    o.TimeSinceStart(tv));
 }
 
 void print_xml_hosts() {
@@ -2529,6 +2518,7 @@ void print_xml_hosts() {
 void printfinaloutput() {
   time_t timep;
   char mytime[128];
+  int err = 0;
   struct timeval tv;
   char statbuf[128];
 
@@ -2537,7 +2527,7 @@ void printfinaloutput() {
 
   if (o.numhosts_scanned == 0
 #ifndef NOLUA
-      && o.scriptupdatedb == 0
+      && !o.scriptupdatedb
 #endif
       )
     error("WARNING: No targets were specified, so 0 hosts scanned.");
@@ -2566,9 +2556,6 @@ void printfinaloutput() {
     log_write(LOG_STDOUT | LOG_SKID, "           %s\n",
               getFinalPacketStats(statbuf, sizeof(statbuf)));
 
-  Strncpy(mytime, ctime(&timep), sizeof(mytime));
-  chomp(mytime);
-
   xml_start_tag("runstats");
   print_xml_finished_open(timep, &tv);
   xml_attribute("exit", "success");
@@ -2578,12 +2565,24 @@ void printfinaloutput() {
   xml_end_tag();
   xml_newline();
 
-  log_write(LOG_NORMAL | LOG_MACHINE,
-            "# Nmap done at %s -- %u %s (%u %s up) scanned in %.2f seconds\n",
-            mytime, o.numhosts_scanned,
-            (o.numhosts_scanned == 1) ? "IP address" : "IP addresses",
-            o.numhosts_up, (o.numhosts_up == 1) ? "host" : "hosts",
-            o.TimeSinceStart(&tv));
+  err = n_ctime(mytime, sizeof(mytime), &timep);
+  if (!err) {
+    chomp(mytime);
+    log_write(LOG_NORMAL | LOG_MACHINE,
+        "# Nmap done at %s -- %u %s (%u %s up) scanned in %.2f seconds\n",
+        mytime, o.numhosts_scanned,
+        (o.numhosts_scanned == 1) ? "IP address" : "IP addresses",
+        o.numhosts_up, (o.numhosts_up == 1) ? "host" : "hosts",
+        o.TimeSinceStart(&tv));
+  }
+  else {
+    log_write(LOG_NORMAL | LOG_MACHINE,
+        "# Nmap done -- %u %s (%u %s up) scanned in %.2f seconds\n",
+        o.numhosts_scanned,
+        (o.numhosts_scanned == 1) ? "IP address" : "IP addresses",
+        o.numhosts_up, (o.numhosts_up == 1) ? "host" : "hosts",
+        o.TimeSinceStart(&tv));
+  }
 
   xml_end_tag(); /* nmaprun */
   xml_newline();
@@ -2722,7 +2721,7 @@ void nmap_adjust_loglevel(bool trace) {
   nsock_set_loglevel(nsock_loglevel);
 }
 
-void nmap_nsock_stderr_logger(const struct nsock_log_rec *rec) {
+static void nmap_nsock_stderr_logger(const struct nsock_log_rec *rec) {
   int elapsed_time;
 
   elapsed_time = TIMEVAL_MSEC_SUBTRACT(rec->time, *(o.getStartTime()));
@@ -2730,3 +2729,7 @@ void nmap_nsock_stderr_logger(const struct nsock_log_rec *rec) {
   log_write(LOG_STDERR, "NSOCK %s [%.4fs] %s(): %s\n", nslog2str(rec->level),
             elapsed_time/1000.0, rec->func, rec->msg);
 }
+
+void nmap_set_nsock_logger() {
+  nsock_set_log_function(nmap_nsock_stderr_logger);
+}
